//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FILE_H
#define FILE_H

#include <string>
#include <list>
#include <SDL.h>
#include <pgdropdown.h>

/** \brief Miscellaneous functions for unified file access
  * 
  * These functions should be the sole way to access any files. They will
  * automatically prepend the correct directory, extract the correct file etc.
  * This enables us to install and package freelords (there is no fixed
  * directory structure). To use these functions, you issue e.g. an armyset name
  * and have the full path returned, which is a file you can then load.
  */

class File
{
    public:
        /** Scan the data directories for armysets
          * 
          * @return a list of available armysets
          */
        static std::list<std::string> scanArmysets();


        /** Get the armyset description file
          *
          * @param armysetname      the name of the armyset
          * @return the full name of the description file
          */
        static std::string getArmyset(std::string armysetname);

        /** Get the armyset picture file
          * 
          * @param amysetname       the name of the armyset
          * @param alpha            if set to true, copy alpha channel
          * @return the surface which contains the army pictures of this armyset
          */
        static SDL_Surface* getArmyPicture(std::string armysetname, bool alpha=true);
        
        /** Get the armyset mask file
          *
          * @param armysetname      the name of the armyset
          * @return the surface which contains the masks of the armies or 0 if
          * the mask file didn't exist.
          */
        static SDL_Surface* getArmyMask(std::string armysetname);

        

        /** Get the description file for the mapset
          * 
          * @param mapsetname       the name of the mapset
          * @return the full name of the description file
          */
        static std::string getMapset(std::string mapsetname);

        /** Get a mapset picture
          * @param mapsetname       the name of the mapset
          * @param picname          the name of the picture
          * @param alpha            if set to true, copy (don't use) alpha channel
          * @param colorkey         if set to true, the surface has a color key
          *                         set to bright pink
          * @return the surface which contains the picture
          */
        static SDL_Surface* getMapsetPicture(std::string mapsetname, std::string picname,
                                                bool alpha=true, bool colorkey=false);

        /** Load a mask needed by various mapset images (e.g. cities)
          * 
          * @param mapsetname       the name of the mapset
          * @param picname          the name of the mask picture
          * @return the surface which contains the surface for the mask
          */
        static SDL_Surface* getMapsetMask(std::string mapsetname, std::string picname);

        
        
        // load misc file/pic
        static std::string getMiscFile(std::string filename);
        static SDL_Surface* getMiscPicture(std::string picname, bool alpha=true);


        
        //! Load the picture of an item
        static SDL_Surface* getItemPicture(std::string picname, bool alpha=true);

        //! Load the xml file describing the items
        static std::string getItemDescription();
        

        //! Get the path to a border tile
        static SDL_Surface* getBorderPic(std::string filename);

        
        //get save game path
        static std::string getSavePath();

        static std::list<std::string> s_armySetList;
        static std::list<std::string> s_tileSetList;

        /** Get the available tilesets and fill a dropdown box with these entries
          * 
          * @param dropdown     the dropdown box to be filled with the information
          * @return a list of available tilesets
          */
        static std::list<std::string> scanTilesets(PG_DropDown* drop_down);

    private:
        static SDL_Surface* loadImage(std::string filename, bool alpha, bool colorkey=true);
};

#endif //FILE_H

// End of file
