//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GAME_SCENARIO_H
#define GAME_SCENARIO_H

#include <paragui.h>
#include <pgeventobject.h>
#include <pgmessageobject.h>
#include <string>
#include <sigc++/object_slot.h>
#include <list>
#include "player.h"
#include "Event.h"

class Server;
class Client;

/** Class to hold several scenario options
  * 
  * This class has two functions. On the one hand side, it holds some data
  * about the current scenario being played (such as the name), on the other
  * hand it has a kind of supervisor function. Loading and saving works in
  * a hierarchical way with superior objects (such as the playerlist) saving
  * their data and then telling inferior objects (such as players) to save
  * their data as well. GameScenario is kind of the root of the saving or
  * loading process. For more information about the saving procedure, have
  * a look at XML_Helper.
  */

class GameScenario : public PG_EventObject, public PG_MessageObject,
                     public SigC::Object
{
    public:
        enum GameType {SCENARIO = 1, SINGLE_PLAYER = 2, MULTI_PLAYER_HOST = 3,
            MULTI_PLAYER_CLIENT = 4};
        enum MapType {RANDOM, LOADED};

        //! Ignore this one, playing over network is not implemented
        GameScenario(GameType gametype, std::string ip, int port);

        /** Load the game scenario using a specified save game
          * 
          * @param savegame     the full name of the savegame to load
          * @param broken       set to true if something goes wrong
          */
        GameScenario(std::string savegame, bool& broken);
        ~GameScenario();

        //! Returns the number of the current turn.
        int getTurns() const {return d_turns;}

        //! Returns the turn mode. See NextTurn for a description.
        bool getTurnmode() {return d_turnmode;}

        //! Returns the name of the scenario.
        std::string getName() {return d_name;}

        //! Returns whether this is a multiplayer game or not.
        bool isMultiPlayer() const;
        
        //! Increments the turn number. The rest is done by NextTurn.
        void nextTurn();

        //! Sets the name of the scenario.
        void setName(std::string name) {d_name = name;}
        
        /** Saves the game. See XML_Helper for further explanations.
          * 
          * @param filename     the full name of the save game file
          * @return true if all went well, false otherwise
          */
        bool saveGame(std::string filename);

        /** Callback function for loading a game. See XML_Helper for details.
          *
          * @param tag      the tag name
          * @param helper   the helper for parsing the save game file
          * @return true if all went well, false otherwise.
          */
        bool load(std::string tag, XML_Helper* helper);

        //! Adds an event to the event list
        void addEvent(Event* event);

        /** Deactivates all events. This becomes neccessary because in certain
          * circumstances several events may be triggered and cause the game to
          * finish. To prevent possible bugs, we deactivate all events on a
          * game end so that they are not triggered.
          */
        void deactivateEvents();

        //! Returns the internal list of events
        std::list<Event*> getEventlist() {return d_events;}

    private:

        //! Does nothing useful now, relic from earlier times.
        void initGame();

        //! Relic from earlier times as well.
        void initMultiPlayer();
        
        // DATA
        int d_turns;
        GameType d_gameType;
        std::string d_name;
        bool d_turnmode; //see NextTurn for a description of this option

        std::list<Event*> d_events;
        
        Server* d_server;
        Client* d_client;
};

#endif // GAME_SCENARIO_H

// End of file
