//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <pgmessagebox.h>
#include "MainWindow.h"
#include "w_edit.h"
#include "defs.h"
#include "Configuration.h"
#include "goldreport.h"
#include "PG_FileDialog.h"
#include "citiesreport.h"
#include "questsreport.h"
#include "stackreport.h"
#include "OptionsDialog.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
//#define debug(x)

MainWindow::MainWindow(GameScenario* gameScenario, const PG_Rect& rect)
    :PG_ThemeWidget(0, rect, true), d_lastsave("")
{
    // create menus
    d_fileMenu = new PG_PopupMenu(0, 20, 0, 0);
    d_fileMenu->addMenuItem(_("Load"), 0, MSG_CALLBACK_OBJ(&MainWindow::file_load), this);
    d_fileMenu->addMenuItem(_("Save"), 1, MSG_CALLBACK_OBJ(&MainWindow::file_save), this);
    d_fileMenu->addMenuItem(_("Save as"), 2, MSG_CALLBACK_OBJ(&MainWindow::file_saveas), this);
    d_fileMenu->addSeparator();
    d_fileMenu->addMenuItem(_("Quit"), 3, MSG_CALLBACK_OBJ(&MainWindow::file_quit), this);

    d_reportsMenu = new PG_PopupMenu(0, 170, 0, 0);
    d_reportsMenu->addMenuItem(_("Armies"), 0, MSG_CALLBACK_OBJ(&MainWindow::reports_armies), this);
    d_reportsMenu->addMenuItem(_("Cities"), 1, MSG_CALLBACK_OBJ(&MainWindow::reports_cities), this);
    d_reportsMenu->addMenuItem(_("Gold"), 2, MSG_CALLBACK_OBJ(&MainWindow::reports_gold), this);
    d_reportsMenu->addMenuItem(_("Quests"), 3, MSG_CALLBACK_OBJ(&MainWindow::reports_quests), this);
    
    d_optionsMenu = new PG_PopupMenu(0, 0, 0, 0);
    d_optionsMenu->addMenuItem(_("Game options"), 0, MSG_CALLBACK_OBJ(&MainWindow::options_game), this);

    d_helpMenu = new PG_PopupMenu(0, 470, 0, 0);
    d_helpMenu->addMenuItem(_("Help"), 0, MSG_CALLBACK_OBJ(&MainWindow::help_help), this);
    d_helpMenu->addSeparator();
    d_helpMenu->addMenuItem(_("About"), 1, MSG_CALLBACK_OBJ(&MainWindow::help_about), this);

    // create menubar
    d_menuBar = new PG_MenuBar(this, PG_Rect(0, 0, rect.w, 25));
    d_menuBar->Add(_("File"), d_fileMenu);
    d_menuBar->Add(_("Reports"), d_reportsMenu);
    d_menuBar->Add(_("Options"), d_optionsMenu);
    d_menuBar->Add(_("Help"), d_helpMenu);
    
    d_w_edit = new W_Edit(gameScenario, this, PG_Rect(rect.x, rect.y + 25, rect.w, rect.h - 25));

    d_fileMenu->Hide();
    d_reportsMenu->Hide();
    d_optionsMenu->Hide();
    d_helpMenu->Hide();
}

MainWindow::~MainWindow()
{
    delete d_menuBar;
    delete d_fileMenu;
    delete d_reportsMenu;
    delete d_optionsMenu;
    delete d_helpMenu;
    delete d_w_edit;
}

void MainWindow::startGame()
{
    d_w_edit->startGame();
}

void MainWindow::loadGame(std::string filename,bool resetfilename)
{
    d_w_edit->loadGame();
    // Andrea : I have added the boolean control value because there where a bug concerning
    // the saving of the tutorial game. Instead of saving it into the .freelords directory
    // it saved it into the ...../share/freelords/map/ directory
    // Now after loading the tutorial map the filename is reset to "" so that it will save 
    // the file into correct path.
    if (resetfilename) d_lastsave="";
    else d_lastsave = filename;
}

void MainWindow::stopGame()
{
    //This function stops all current game activity. The background is that
    //MainWindow doesn't automatically know that a game has ended, so it has
    //to be told this fact.
    d_w_edit->stopGame();
}

// This event handler is from Thomas Plonka
// Scrolling of the map with the keybord arrows and reports with the keyboard.
// It is a paragui event and no't a SDL event.
bool MainWindow::eventKeyDown(const SDL_KeyboardEvent* key)
{	
	// Check , which key was pressed and change the variables
	// and the position of the viewrect , or start reports	
	switch (key->keysym.sym)
	{		
	    case SDLK_ESCAPE:
			d_w_edit->unselectStack();
		    break; 			
		case SDLK_LEFT:
			arrowx = -1;
			break;
		case SDLK_RIGHT:
			arrowx = 1;
			break;
		case SDLK_UP:
			arrowy = -1;
			break;
		case SDLK_DOWN:
			arrowy = 1;
			break;
		case SDLK_a:	
            reports_armies(0,0,0,0);
			break;
		case SDLK_c:	
            reports_cities(0,0,0,0);
			break;
		case SDLK_g:
            reports_gold(0,0,0,0);
			break;
		case SDLK_l:
            file_load(0,0,0,0);
			break;
		case SDLK_q:
            reports_quests(0,0,0,0);
			break;
		case SDLK_s:
            file_saveas(0,0,0,0);
			break;
		default:
			break;	
	}
	// connection to smallmap
	d_w_edit->helpSmallmap(arrowx, arrowy);
	
	// If "arrowx" and "arrowy" are not set to "0" ,
	// diagonal scrolling with the keyboard-arrows
	arrowx = 0;
	arrowy = 0;
					
	return true;	
}


PARAGUI_CALLBACK(MainWindow::file_load)
{
	//to make sure no timers interfere with the graphics painting routines
    d_w_edit->stopTimers();

    //if (PlayerReallyWantsToLoadGame)  (to be implemented later)
    PG_FileDialog opendialog(0, PG_FileDialog::PG_OPEN, true, Configuration::s_savePath,
            "*.sav", _("Freelords savegames"));
    opendialog.Show();
    opendialog.RunModal();


    std::string savefile = opendialog.getSelectedFile();

    if (savefile.empty()) return true;

    //we delete d_w_edit only here to have a nice background during the
    //open dialog :)
    delete d_w_edit;

    bool broken;
    GameScenario* gameScenario = new GameScenario(savefile, broken);

    if (broken)
    {
        std::cerr <<_("Couldn't load savefile ") <<savefile <<_(". Bailing out.\n");
        exit(-1);
    }

    d_w_edit = new W_Edit(gameScenario, this,
                    PG_Rect(my_xpos, my_ypos + 25, my_width, my_height - 25));
    d_w_edit->startTimers();
    d_w_edit->Show();
    d_w_edit->loadGame();

    d_lastsave = savefile;

    return true;
}

PARAGUI_CALLBACK(MainWindow::file_save)
{
    if (d_lastsave.empty())
    {
        file_saveas(0,0,0,0);   // open dialog for file selection etc.
        return true;
    }

    // tell GameScenario to save the game
    bool success = d_w_edit->save(d_lastsave);

    //timers interfere with the graphics drawing routine
    d_w_edit->stopTimers();

    if (success)
    {
        PG_MessageBox mb(this, PG_Rect(200, 200, 200, 150), _("Save"),
                _("Game Saved"), PG_Rect(60, 110, 80, 30), _("OK"));
        mb.Show();
        mb.WaitForClick();
        mb.Hide();
    }
    else
    {
        PG_MessageBox mb(this, PG_Rect(200, 200, 200, 150), _("Save"),
                _("Error saving. Game not saved"),
                PG_Rect(60, 110, 80, 30), _("OK"));
        mb.Show();
        mb.WaitForClick();
        mb.Hide();
    }

    d_w_edit->startTimers();

    return true;
}

PARAGUI_CALLBACK(MainWindow::file_saveas)
{
    //temporarily stop all timers
    d_w_edit->stopTimers();

    PG_FileDialog savedialog(0, PG_FileDialog::PG_SAVE, true, Configuration::s_savePath,
            "*.sav", _("Freelords savegames"));
    savedialog.Show();
    savedialog.RunModal();
    string savefile = savedialog.getSelectedFile();

    //nothing selected
    if (savefile.empty()) return true;

    //tell GameScenario to save the game
    bool success = d_w_edit->save(savefile);
    
    if (success)
    {
        PG_MessageBox mb(this, PG_Rect(200, 200, 200, 150), _("Save"),
                _("Game Saved."),
                PG_Rect(60, 110, 80, 30), _("OK"));
        mb.Show();
        mb.WaitForClick();
        mb.Hide();

        d_lastsave = savefile;
    }
    else
    {
        PG_MessageBox mb(this, PG_Rect(200, 200, 200, 150), _("Save"),
                _("Error saving. Game not saved"),
                PG_Rect(60, 110, 80, 30), _("OK"));
        mb.Show();
        mb.WaitForClick();
        mb.Hide();
    }

    d_w_edit->startTimers();

    return true;
}

PARAGUI_CALLBACK(MainWindow::file_quit)
{
    d_w_edit->stopTimers();
    squitting.emit();
    return true;
}

PARAGUI_CALLBACK(MainWindow::reports_armies)
{
    debug("Showing army report")

    d_w_edit->stopTimers();

    StackReport report(this, PG_Rect(Width()/2 - 250, Height()/2 - 200, 530, 400));

    report.Show();
     //some parts of the report which we don't want anyone to see have been
    //displayed by calling the show event. We simulate a user action here
    //to redraw the whole thing.
    report.b_upClicked(0, 0, 0, 0);
    report.RunModal();
    report.Hide();

    d_w_edit->startTimers();

    return true;
}

PARAGUI_CALLBACK(MainWindow::reports_quests)
{
    debug("Showing quests report")

    d_w_edit->stopTimers();

    QuestsReport report(this, PG_Rect(Width()/2 - 250, Height()/2 - 200, 
                        535, 400));
    report.Show();
     //some parts of the report which we don't want anyone to see have been
    //displayed by calling the show event. We simulate a user action here
    //to redraw the whole thing.
    report.b_upClicked(0, 0, 0, 0);
    report.RunModal();
    report.Hide();

    d_w_edit->startTimers();
    return true;
}

PARAGUI_CALLBACK(MainWindow::reports_cities)
{
    debug("Showing city report")

    d_w_edit->stopTimers();

    CitiesReport report(this, PG_Rect(Width()/2 - 250, Height()/2 - 200, 530, 400));

    report.Show();
     //some parts of the report which we don't want anyone to see have been
    //displayed by calling the show event. We simulate a user action here
    //to redraw the whole thing.
    report.b_upClicked(0, 0, 0, 0);
    report.RunModal();
    report.Hide();

    d_w_edit->startTimers();

    return true;
}

PARAGUI_CALLBACK(MainWindow::reports_gold)
{
    debug("Showing gold report")

    d_w_edit->stopTimers();

    GoldReport report(this, PG_Rect(Width()/2 - 150, Height()/2 - 200,
                    300, 300));
    report.Show();
    report.RunModal();
    report.Hide();

    d_w_edit->startTimers();

    return true;
}

PARAGUI_CALLBACK(MainWindow::options_game)
{
    d_w_edit->stopTimers();

    OptionsDialog opts(this, PG_Rect(Width()/2 - 150, Height()/2 - 240, 260, 200));

    opts.Show();
    opts.RunModal();
    opts.Hide();

    d_w_edit->startTimers();

    return true;
}

PARAGUI_CALLBACK(MainWindow::help_help)
{
    std::cerr << _("not implemented yet\n");
    return true;
}

PARAGUI_CALLBACK(MainWindow::help_about)
{
    std::cerr << _("not implemented yet\n");
    return true;
}

// End of file
