//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MAPGENERATOR_H
#define MAPGENERATOR_H

#include <string>
#include <vector>


class portneeded;



typedef  std::vector<portneeded> vecports;

typedef  std::vector<vecports> vecs2d;




/** This class creates a map including the buildings (cities, ruins, temples).
  * It does NOT care about player setup and such. This is done in a second step
  * by the CreateScenario class. The parameters for map creation are set by
  * GamePreferencesDialog::fillData().
  * GamePreferencesDialog does use the MapConfDialog but this is
  * just a skeleton - GamePreferencesDialog does all the work.
  *
  * The data is stored in two variables:
  *   - the terrain map stores the terrain information
  *   - the building map stores the information about building placement
  *
  * For historical reasons *cough*, the maps are long character arrays. The
  * definition of the single characters can be found in defs.h; missing
  * buildings are denoted by a space. As a normal user, you will propably never
  * need to worry about the format anyway. It is read and "translated" by the
  * CreateScenario class which kind of supervises the scenario creation.
  *
  * Map generators with different styles (e.g. islands), derived from
  * this class are possible and hopefully this will be done in the
  * future. 
  * 
  * When designating sites for cities it takes care to ensure that 
  * the first city sites on any land mass will be ports on those
  * seas that connect with other land masses and islands.
  * In so doing it creates two data structures that may be useful
  * elsewhere.  
  *  First it makes a map of all the separate land areas
  * and seas.  This is stored in the "array d_l_mass" which has a 
  * similar format to the terain map.  Each location has the id of
  * land/sea to which the tile is part of.  All seas have an
  * id less than zero and all land masses have an id greater than
  * zero.
  *  The other potentially useful structure contains all the 
  * routes between landmasses to each other includeing the id
  * of the sea that must be crossed to get to the land mass 
  * with the destination id.  This is currently stored in a
  * vector
  *
  */




class MapGenerator
{
private:
    int d_xdir[8];
    int d_ydir[8];
    public :
    
      
        MapGenerator();
        ~MapGenerator();

        //! Set the number of cities
        int setNoCities(int nocities);

        //! Set the number of ruins
        int setNoRuins(int noruins);

        //! Set the number of temples
        int setNoTemples(int notemples);

        /** Set terrain distribution. If the sum of the percentages is less than
          * 100, the rest is filled with grass.
          */
        void setPercentages(int pwater, int pforest, int pswamp,
                            int phills, int pmountains);

        //! Get number of cities
        int getNoCities() {return d_nocities;}

        //! Get number of ruins
        int getNoRuins() {return d_noruins;}

        //! Get number of temples
        int getNoTemples() {return d_notemples;}
        
        /** Creates a map
          * 
          * Use this function to start off the map generation. This
          * implementation will first distribute the terrain in random patches,
          * normalize it and then distribute the buildings.
          *
          * @param size         the size of the map to be generated
          */
        void MakeMap(int size);

        /** Get the array for the terrain map (shallow copy)
          * 
          * @param size         is set to the size of the generated map
          * @return char array which represents the terrain map
          */
        char* getMap(int& size);
        
        /** Get the buildings map (shallow copy)
          * 
          * @param size         set to the size of the generated map
          * @return char array which represents the building map
          */
        char* getBuildings(int& size);

    private:

        //! Fills the terrain map with grass
        void makePlains();

        //! Finds the nearest bit of terrain so that makeTerrain can
        //! further expand the active block of terrain c
        int seekPlain(char c, int& x,int& y);

        //! Spreads the terrain c with the given percentage over the terrain map
        void makeTerrain(char c, int percent, int contin);

        

        /** Designates the places where the cities will be by setting characters 
          * on the building map.  Creates an island if it
          * finds only water or occupied places.
          *
          * @param cities       the number of cities to distribute
          */
        void makeCities(int cities);
        
        //sets the bulding map, sets to grass and increments city_count
        void putcity(int x,int y, int&  city_count);

        /** Designates the location of buildings across the map. Creates an island 
          * if there is only water or occupied places.
          *
          * @param c            the character for the building
          * @param building     the number of buildings to distribute
          */
        void makeBuildings(char c, int building);
        
        bool tryToPlaceCity(int px,int py, int& city_count,bool& hasPort);
        
        void placePorts(int x,int y, int& city_count);

        //! Returns true if position (x,y) is free for buildings
        bool canPutBuilding(int x,int y);
        
        // moves along coast in a clock wise direction       
        bool walkCoast(int& x,int& y, int sea_id, int dist, int& lastwat);
        
        bool walkCoastAntiClock(int& x,int& y, int sea_id, int dist, int& lastwat);
        
        //! Returns true if position (x,y) is free for a city
        bool canPutCity(int x,int y);

        //! Normalizes the terrain map by removing isolated tiles by chance
        void normalize();

        //! We need to designate seperate land masses so that later we can
        // ensure that the first city on a land mass is a port
        void continents(int& nmrLands, int&  nmrSeas ) ;
        
        void findRoutes(int nmrLands ,int nmrSeas);

        //Data
        char* d_terrain; // the map of the terrain
        char* d_building;
        int* d_l_mass;//  this maps out seperate seas and land areas
        int d_size; // length of one side of the square of the map
        int d_pswamp, d_pwater, d_pforest, d_phills, d_pmountains;
        int d_conwat;//continuity of water area
        unsigned int d_nocities, d_notemples, d_noruins;
        vecs2d d_portneed;//the seas a landmass needs to have a port on
};

#endif

// End of file
