//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef __QUEST_H
#define __QUEST_H

#include <string>
#include <queue>
#include "xmlhelper.h"
#include "hero.h"
#include "stack.h"

class QuestsManager;

/** \brief Second auxiliary function - getHeroById 
 *
 *  Additionally it allows to get the Hero's stack.
 */
 Hero* getHeroById(unsigned int id, Stack** stack = NULL);

/**
 * \note Alas the quest cannot disconnect itself when (during processing
 *       of the signal) it detects that it has been marked as 'not pending'
 *       anymore.
 *
 * \todo Add saving common data of quests.
 */
class Quest 
{

    public:

        /** \brief Constructor
         *
         */
        Quest(QuestsManager& q_mgr, int heroId) : pending(1),
                                                  d_q_mgr(q_mgr),
                                                  d_heroId (heroId) {}

    virtual ~Quest() {};
        /** \brief Notification that the quest is to be deleted.
         */
        void deactivate();

        /** \brief Return the description of the quest 
         *
         *  This is the 'static' part of the quest description,
         *  set once the quest has been initialized. Another,
         *  dynamic part consists of the quest's status info,
         *  which is obtained by the getProgress method.
         */
        const char* getDescription() { return d_description.c_str(); }

        /**
         * \brief Get progress information 
         *
         * \param s here we append the progress information
         */
        virtual void getProgress(std::string& s) = 0;

        /**
         * \brief Provide the lines of the message describing
                  the quest completion.
         */
        virtual void getSuccessMsg(std::queue<std::string>& msgs) = 0;

        /**
         * \brief Provide the lines of the message describing
                  the quest completion.
         */
        virtual void getExpiredMsg(std::queue<std::string>& msgs) = 0;

        /** save the quest */
        virtual bool save(XML_Helper* helper) = 0;

        /** \brief Get the id of the hero owning this quest */
        unsigned int getHeroId() { return d_heroId; }

    protected:

        /** it is to be called by children when the quest is finished */
        int questCompleted();
        /** it is to be called by children when the quest is expired for some reason */
        int questExpired();

        /** \brief Text description of the quest 
         *
         * This is to be filled by the children (specific quests).
         */
        std::string d_description;

        /** \brief is this quest still pending? 
         *
         *  By checking value of this field the children find out
         *  whether they should process their signals or return
         *  immediately.
         */
        char pending;

    private:

        /** \brief Reference to the QuestManager. */
        QuestsManager& d_q_mgr;

        /** \brief id of the hero owning this quest */
        const int d_heroId;
};

#endif
