//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "Threat.h"
#include "defs.h"
#include "stacklist.h"
#include <iostream>
#include "city.h"
#include "stack.h"
#include "player.h"
#include "AI_Analysis.h"

using namespace std;

Threat::Threat(City *c)
    :d_city(c), d_ruin(0)
{
    d_stacks = new Stacklist();
    d_player = d_city->getPlayer();
}

Threat::Threat(Stack *s)
    :d_city(0), d_ruin(0)
{
    d_stacks = new Stacklist();
    d_stacks->push_front(s);
    d_player = s->getPlayer();
}

Threat::Threat(Ruin *r)
    :d_city(0), d_ruin(r), d_player(0)
{
    d_stacks = new Stacklist();
}

Threat::~Threat()
{
    delete d_stacks;
}

std::string const Threat::toString()
{
    if (d_city)
    {
        return d_city->getName() + " owned by " + d_player->getName();
    }
    else if (d_ruin)
    {
        return d_ruin->getName();
    }
    else
    {
        return "stack owned by " + d_player->getName();
    }
}

bool const Threat::near(PG_Point pos, Player *p)
{
    if (p != d_player) return false;
    if (d_city)
    {
        return d_city->contains(pos);
    }
    else if (d_ruin)
    {
        return d_ruin->contains(pos);
    }
    else
    {
        for (Stacklist::iterator it = d_stacks->begin(); it != d_stacks->end(); ++it)
        {
            Stack *s = *it;
            if (abs(pos.x - s->getPos().x) <= 1 && abs(pos.y - s->getPos().y <= 1)) return true;
        }
        return false;
    }
}

void Threat::addStack(Stack *stack)
{
    d_stacks->push_back(stack);
}

// this is the strength of the threat to us
float const Threat::strength()
{
    // neutral cities pose no threat
    if (d_city)
        if (d_city->getPlayer()->getType() == Player::NEUTRAL)
            return 0.0;
    float score = 0.0;
    for (Stacklist::iterator it = d_stacks->begin(); it != d_stacks->end(); ++it)
    {
        Stack *s = *it;
        score += AI_Analysis::assessStackStrength(s);
    }
    return score;
}

float const Threat::value()
{
    float score = 0.0;
    // if city has turned friendly, it is no longer a valuable target
    if (d_city)
        if (d_city->getPlayer() != d_player && !d_city->isBurnt())
            score += 10.0;
    // if ruin has become searched, it is no longer valuable
    if (d_ruin)
        if (!d_ruin->isSearched())
            score += 5.0;
    score += d_danger;
    return score;
}

// memory is owned by the caller
// can return 0 if the threat has been destroyed
PG_Point *Threat::getClosestPoint(PG_Point location)
{
    PG_Point *result = 0;
    if (d_city)
    {
        PG_Point cpos = d_city->getPos();
        result = new PG_Point(cpos);
        if (location.x > result->x) result->x++;
        if (location.y > result->y) result->y++;
    }
    else if (d_ruin)
    {
        PG_Point rpos = d_ruin->getPos();
        result = new PG_Point(rpos);
    }
    else
    {
        int distToClosest = 1000000;
        for (Stacklist::iterator it = d_stacks->begin(); it != d_stacks->end(); ++it)
        {
            Stack *s = *it;
            PG_Point spos = s->getPos();
            //UL: remember, stacks can move diagonally
            int dist = abs(spos.x - location.x);
            int disty = abs(spos.y - location.y);
            if (dist > disty)
                dist = disty;
            if (dist < distToClosest)
            {
                if (!result) result = new PG_Point();
                distToClosest = dist;
                result->x = spos.x;
                result->y = spos.y;
            }
        }
    }
    return result;
}

void Threat::deleteStack(Stack* s)
{
    d_stacks->remove(s);
}
