//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include <algorithm>
#include <map>
#include "ai_smart.h"
#include "playerlist.h"
#include "armysetlist.h"
#include "FightDialog.h"
#include "stacklist.h"
#include "citylist.h"
#include "Threat.h"
#include "Threatlist.h"
#include <fstream>
#include "path.h"
#include "AI_Analysis.h"
#include "AI_Allocation.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
#define debug(x)

AI_Smart::AI_Smart(string name, string armyset, SDL_Color color)
    :RealPlayer(name, armyset, color, Player::AI_SMART)
{
    snewlevelAiArmy.connect(SigC::slot((*this), &AI_Smart::newLevelArmy));
}

AI_Smart::AI_Smart(const Player& player)
    :RealPlayer(player)
{
    snewlevelAiArmy.connect(SigC::slot((*this), &AI_Smart::newLevelArmy));
}

AI_Smart::AI_Smart(XML_Helper* helper)
    :RealPlayer(helper)
{
    snewlevelAiArmy.connect(SigC::slot((*this), &AI_Smart::newLevelArmy));
}

AI_Smart::~AI_Smart()
{
}

bool AI_Smart::startTurn()
{
    debug(getName() << " start_turn")
    clearActionlist();

    // the real stuff
    examineCities(Citylist::getInstance());
    int loopCount = 0;
    while (true)
    {
        // if the code is working, this loop will eventually terminate.
        // However, in case there is a bug somewhere, we don't want the AI players to move forever,
        // so loopCount is a safeguard to prevent that.
        loopCount++;
        if (loopCount >= 5) break;
        AI_Analysis *analysis = new AI_Analysis(this);
        AI_Allocation *allocation = new AI_Allocation(analysis, this);
        int moveCount = allocation->move();
        // tidying up
        delete analysis;
        analysis = 0;
        delete allocation;
        allocation = 0;
        // stop when no more stacks move
        if (moveCount == 0) break;
    }
    d_stacklist->setActivestack(0);
    return true;
}

void AI_Smart::newLevelArmy(Army* a)
{
    // always increase the strength of the army
    a->setOrigStrength((a->getOrigStrength()+1));
}

bool AI_Smart::invadeCity(City* c)
{
    // always occupy an enemy city
    bool retval = cityOccupy(c);
    sinvadingCity.emit(c);
    soccupyingCity.emit(c);

    // Update its production
    maybeBuyProduction(c);
    setBestProduction(c);
    return retval;
}

bool AI_Smart::recruitHero(Hero* hero, int cost)
{
    return true;    //always recruit heroes
}

int AI_Smart::maybeBuyProduction(City *c)
{
    int armytype = chooseArmyTypeToBuy(c);
    if (armytype >= 0 && cityBuyProduction(c, armytype))
    {
        return armytype;
    }
    else
    {
        return -1;
    }
}

int AI_Smart::setBestProduction(City *c)
{
    int select = -1;
    int score = -1;

    // we try to determine the most attractive production
    for (int i = 0; i < 4; i++)
    {
        if (c->getArmytype(i) == -1)    // no production in this slot
            continue;

        Armyset_army *proto = c->getArmyset_army(i);
        if (scoreArmyType(proto) > score)
        {
            select = i;
            score = scoreArmyType(proto);
        }
    }

    // this should work in any case; if no production exists, we set it to none
    if (select != c->getProduction())
    {
        cityChangeProduction(c, select);
        debug(getName() << " Change production to " << select << " in " << c->getName())
    }
    return c->getProduction();
}

int AI_Smart::chooseArmyTypeToBuy(City *c)
{
    int i, bestScore, bestIndex;
    if (c->getProduction() > 0)
    {
        return -1;
    }
    Armyset *armySet = getArmyset();
    bestScore = -1;
    bestIndex = -1;
    // you can't buy a hero type!
    for (i=armySet->getSize()-1; i >= 2; i--)
    {
        Armyset_army *proto = &armySet->army[i];
        int score = scoreArmyType(proto);
        if (proto->getOrigArmyBonus() & ARMY_SHIP) continue;
        if (proto->getProductionCost() > d_gold) continue;
        if (score >= bestScore)
        {
            bestIndex = i;
            bestScore = score;
        }
    }
    return bestIndex;
}

int AI_Smart::scoreArmyType(Armyset_army *a)
{
    // UL: I have spent some minutes calculating the score of the default
    // armyset. The algorithm looks reasonable.
    int strength = a->getOrigStrength() + a->getOrigDefense() + a->getOrigMaxHp() / 3;
    return (strength - a->getProduction());
}

void AI_Smart::examineCities(Citylist *allCities)
{
    for (Citylist::iterator it = allCities->begin(); it != allCities->end(); ++it)
    {
        City *city = &(*it);
        if (city->isFriend(this))
        {
            maybeBuyProduction(city);
            setBestProduction(city);
        }
    }
}

// End of file
