//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream>
#include <fcntl.h>
#include <unistd.h>
#include <expat.h>
#include <SDL_image.h>
#include <SDL.h>
#include <pgapplication.h>
#include "armysetlist.h"
#include "armyset.h"
#include "armyset_army.h"
#include "File.h"

#define BUFF_SIZE 8192

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
//#define debug(x)

Armysetlist* Armysetlist::s_instance = 0;

Armysetlist* Armysetlist::getInstance()
{
    if (!s_instance)
        s_instance = new Armysetlist();

    return s_instance;
}

void Armysetlist::deleteInstance()
{
    if (s_instance)
        delete s_instance;

    s_instance = 0;
}

Armysetlist::Armysetlist()
    :map<string, Armyset*>()
{
}

Armysetlist::~Armysetlist()
{
}

Armyset* Armysetlist::getArmyset(string name, bool load)
{
    // this one is rather easy thanks to the STL
    // if the name entry doesn't exist, map creates one and initialises with 0

    Armyset* retval = (*this)[name];

    //If it doesn't exist, load the armyset if this is requested.
    if (!retval && load)
    {
        loadArmyset(name);
        retval = (*this)[name];
    }

    return retval;
}

bool Armysetlist::loadGlobalStuff(std::string tag, XML_Helper* helper)
{
    int size;
    bool retval = true;

    //Currently, the only global option is the size of the armyset
    retval &= helper->getData(size, "size");
    //This one causes bugs. Review it!
    (*this)[d_loadName]->setSize(size);

    return retval;
}

bool Armysetlist::loadArmy(string tag, XML_Helper* helper)
{
    string name;
    int production;
    int productionCost;
    int upkeep;
    int strength;
    int defense;
    int hitpoints;
    int expvalue;
    int moves;
    int vitality;
    unsigned int moveBonus;
    unsigned int armyBonus;

    helper->getData(name, "name");
    helper->getData(production, "production");
    helper->getData(productionCost, "production_cost");
    helper->getData(upkeep, "upkeep");
    helper->getData(strength, "strength");
    helper->getData(defense, "defense");
    helper->getData(hitpoints, "hitpoints");

    helper->getData(expvalue, "expvalue");

    helper->getData(moves, "moves");
    helper->getData(vitality, "vitality");
    helper->getData(moveBonus, "move_bonus");
    helper->getData(armyBonus, "army_bonus");

    Armyset_army army;
    army.setName(name);
    army.setProduction(production);
    army.setProductionCost(productionCost);
    army.setUpkeep(upkeep);
    army.setOrigStrength(strength);
    army.setOrigDefense(defense);
    army.setOrigMaxHp(hitpoints);

    army.setXpReward(expvalue);

    army.setOrigMaxMoves(moves);
    army.setOrigVitality(vitality);
    army.setOrigMoveBonus(moveBonus);
    army.setOrigArmyBonus(armyBonus);
    (*this)[d_loadName]->army.push_back(army);

    //army.printDebugInfo();

    d_loadCounter++;

    return true;
}

bool Armysetlist::loadArmyset(string name)
{
    debug("load_armyset()");

    //to check if we have loaded the same number of armysets than the one
    //given in the armyset description file
    d_loadCounter = 0;
    d_loadName = name;

    //first, look if Armyset already exists
    if ((*this)[name] != 0)
    {
        return true;
    }

    //no, so create a new Armyset first
    (*this)[name] = new Armyset(name);

    XML_Helper helper(File::getArmyset(name), ios::in, false);
    helper.registerTag("armyset", SigC::slot((*this), &Armysetlist::loadGlobalStuff));
    helper.registerTag("army", SigC::slot((*this), &Armysetlist::loadArmy));
    helper.parse();

    if ((*this)[name]->getSize() != d_loadCounter)
    {
        std::cerr <<"while loading armyset " <<d_loadName;
        std::cerr <<" an error occured\nThe number of armysets differs from";
        std::cerr <<"the given size. Exiting...\n";
        exit(-1);
    }

    // set the pixmaps of the army (split the armyGallery)
    SDL_Surface* armyGallery = File::getArmyPicture(d_loadName);
    SDL_PixelFormat* fmt = armyGallery->format;
    // don't use alpha information, just copy the channel! very important
    SDL_SetAlpha(armyGallery, 0, 0);
    for (int i = 0; i < d_loadCounter; i++)
    {
        SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_HWSURFACE,
                40, 40, fmt->BitsPerPixel, fmt->Rmask, fmt->Gmask,
                fmt->Bmask, fmt->Amask);

        SDL_BlitSurface(armyGallery, PG_Rect(40*i, 0, 40, 40).SDLRect(), tmp, 0);
        SDL_Surface* pixmap = SDL_DisplayFormatAlpha(tmp);
        SDL_FreeSurface(tmp);
        (*this)[d_loadName]->army[i].setPixmap(pixmap);
    }

    // not needed anymore
    SDL_FreeSurface(armyGallery);
   
   
    //Second, load the masks if they exist
    armyGallery = File::getArmyMask(d_loadName);
    
    //no mask file, so ignore it
    if (!armyGallery)
        return true;
    
    fmt = armyGallery->format;

    //The mask surface should _always_ have a 256 or lesser color palette. If
    //this isn't the case, the mask pictures are not valid. See GraphicsCache 
    //for more details how they are applied.
    if (fmt->BitsPerPixel > 8)
        return true;

    for (int i = 0; i < d_loadCounter; i++)
    {
        //most of the format information is simply not needed
        SDL_Surface* maskimage;
        maskimage = SDL_CreateRGBSurface(SDL_SWSURFACE, 40, 40,
                                    fmt->BitsPerPixel, 0, 0, 0, 0);

        //since we have 256 color pics, set the palette, else we end up with
        //a monochromatic surface!!
        SDL_SetColors(maskimage, fmt->palette->colors,0, fmt->palette->ncolors);
        
        SDL_BlitSurface(armyGallery, PG_Rect(40*i, 0, 40, 40).SDLRect(),
                        maskimage, 0);
        (*this)[d_loadName]->army[i].setMask(maskimage);
   }

    SDL_FreeSurface(armyGallery);
    
    return true;
}

bool Armysetlist::eraseByName(string name)
{
    iterator it = find(name);
    //armyset exists
    if (it != end())
    {
    //    delete (*this)[name];
        erase(it);
        return true;
    }
    return false;
}

